#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import javax.annotation.concurrent.Immutable;
import net.java.truelicense.core.*;
import net.java.truelicense.json.V2JsonLicenseManagementContext;
import net.java.truelicense.obfuscate.*;

/**
 * A licensing schema for consuming ${subject} license keys
 * with a Free Trial Period (FTP) of thirty calendar days.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public final class LicensingSchema {

    /** Returns the license consumer context. */
    public static LicenseConsumerContext context() { return Lazy.context; }

    /** Returns the license consumer manager. */
    public static LicenseConsumerManager manager() { return Lazy.manager; }

    private static class Lazy {

        @Obfuscate
        static final String SUBJECT = "${subject}";

        @Obfuscate
        static final String PUBLIC_KEY_STORE_NAME = "public.jceks";

        @Obfuscate
        static final String PUBLIC_KEY_STORE_TYPE = "JCEKS";

        static final ObfuscatedString PUBLIC_KEY_STORE_PASSWORD =
                new ObfuscatedString(new long[] {
                    0x66c1016da18a6ffdl, 0xbda53ccf9e30c9efl }); /* => "test1234" */

        @Obfuscate
        static final String PUBLIC_CERT_ENTRY_ALIAS = "mykey";

        @Obfuscate
        static final String PBE_ALGORITHM = "PBEWithSHA1AndDESede";

        static final ObfuscatedString PBE_PASSWORD =
                new ObfuscatedString(new long[] {
                    0xc9c00fc60c9db6b8l, 0x4a19910b12cd00c4l }); /* => "test1234" */

        @Obfuscate
        static final String FTP_KEY_STORE_NAME = "ftp.jceks";

        @Obfuscate
        static final String FTP_KEY_STORE_TYPE = "JCEKS";

        static final ObfuscatedString FTP_KEY_STORE_PASSWORD =
                new ObfuscatedString(new long[] {
                    0x7b4abf4aed98b47al, 0xb1e13b4bc0854bccl }); /* => "test1234" */

        @Obfuscate
        static final String FTP_KEY_ENTRY_ALIAS = "mykey";

        static final ObfuscatedString FTP_KEY_ENTRY_PASSWORD =
                new ObfuscatedString(new long[] {
                    0x149d045402a96977l, 0xa448f2162811f378l }); /* => "test1234" */

        static final int FTP_CALENDAR_DAYS = 30;

        static final LicenseConsumerContext context;
        static final LicenseConsumerManager manager;

        static {
            context = new V2JsonLicenseManagementContext(SUBJECT).consumer();
            /*manager = context.ftpManager(
                context.manager(
                    context.keyStore(
                        context.resource(PUBLIC_KEY_STORE_NAME),
                        PUBLIC_KEY_STORE_TYPE,
                        PUBLIC_KEY_STORE_PASSWORD,
                        PUBLIC_CERT_ENTRY_ALIAS),
                    context.pbe(
                        PBE_ALGORITHM,
                        PBE_PASSWORD),
                    context.userNodeForPackage(Main.class)),
                context.ftpKeyStore(
                    context.resource(FTP_KEY_STORE_NAME),
                    FTP_KEY_STORE_TYPE,
                    FTP_KEY_STORE_PASSWORD,
                    FTP_KEY_ENTRY_ALIAS,
                    FTP_KEY_ENTRY_PASSWORD),
                null,
                context.userNodeForPackage(${classInPackageForUserPreferencesNodeForStoringAutoCreatedLicenseKeyForFreeTrialPeriod}.class),
                FTP_CALENDAR_DAYS);*/
            manager = context
                .manager()
                    .parent()
                        .keyStore()
                            .loadFromResource(PUBLIC_KEY_STORE_NAME)
                            .storeType(PUBLIC_KEY_STORE_TYPE)
                            .storePassword(PUBLIC_KEY_STORE_PASSWORD)
                            .alias(PUBLIC_CERT_ENTRY_ALIAS)
                            .inject()
                        .pbe()
                            .algorithm(PBE_ALGORITHM)
                            .password(PBE_PASSWORD)
                            .inject()
                        .storeInUserNodeForPackage(Main.class)
                        .inject()
                    .keyStore()
                        .loadFromResource(FTP_KEY_STORE_NAME)
                        .storeType(FTP_KEY_STORE_TYPE)
                        .storePassword(FTP_KEY_STORE_PASSWORD)
                        .alias(FTP_KEY_ENTRY_ALIAS)
                        .keyPassword(FTP_KEY_ENTRY_PASSWORD)
                        .inject()
                    .storeInUserNodeForPackage(${classInPackageForUserPreferencesNodeForStoringAutoCreatedLicenseKeyForFreeTrialPeriod}.class)
                    .ftpDays(FTP_CALENDAR_DAYS)
                    .build();
        }
    }
}
