#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import java.io.File;
import static java.lang.System.*;
import java.util.*;
import javax.annotation.concurrent.Immutable;
import javax.swing.*;
import net.java.truelicense.core.*;
import net.java.truelicense.swing.LicenseWizard;

/**
 * Installs, views, verifies or uninstalls license keys for ${subject}.
 * A real application should do something useful besides managing license keys.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public enum Main {

    USAGE {
        @Override protected void run(final Deque<String> params) {
            throw new IllegalArgumentException();
        }
    },

    HELP {
        @Override protected void run(final Deque<String> params) {
            out.println(Main.valueOf(params.pop().toUpperCase(Locale.ROOT)).help());
        }
    },

    INSTALL {
        @Override protected void run(Deque<String> params) throws Exception {
            out.println(cm.install(cc.file(new File(params.pop()))));
        }
    },

    UNINSTALL {
        @Override protected void run(Deque<String> params) throws Exception {
            cm.uninstall();
        }
    },

    VERIFY {
        @Override protected void run(Deque<String> params) throws Exception {
            cm.verify();
        }
    },

    VERSION {
        @Override protected void run(final Deque<String> params) {
            out.println(message("version", Main.class.getSimpleName()));
        }
    },

    VIEW {
        @Override protected void run(Deque<String> params) throws Exception {
            out.println(cm.view());
        }
    },

    WIZARD {
        @Override protected void run(Deque<String> params) throws Exception {
            SwingUtilities.invokeAndWait(new Runnable() {
                @Override public void run() {
                    final LicenseWizard wizard = new LicenseWizard(cm);
                    wizard.setUninstallButtonVisible(true);
                    wizard.showModalDialog();
                }
            });
        }
    };

    private static final LicenseConsumerContext cc = LicensingSchema.context();
    private static final LicenseConsumerManager cm = LicensingSchema.manager();

    /**
     * Runs this command.
     * Implementations are free to modify the given deque.
     *
     * @param  params the command parameters.
     */
    // This method needs to be protected or otherwise ProGuard 4.8 may generate
    // invalid code when --repackageclasses is set, even if
    // -allowaccessmodification is set, too.
    // See https://sourceforge.net/p/proguard/bugs/367/ for details.
    protected abstract void run(Deque<String> params) throws Exception;

    @SuppressWarnings("CallToThreadDumpStack")
    public static void main(final String[] args) {
        int status;
        try {
            final Deque<String> p = new LinkedList<String>(Arrays.asList(args));
            final String cmd = p.pop().toUpperCase(Locale.ROOT);
            valueOf(cmd).run(p);
            status = 0;
        } catch (final IllegalArgumentException ex) {
            errUsage();
            status = 1;
        } catch (final NoSuchElementException ex) {
            errUsage();
            status = 1;
        } catch (Exception ex) {
            ex.printStackTrace();
            status = 2;
        }
        System.exit(status);
    }

    private static void errUsage() {
        final StringBuilder builder = new StringBuilder(25 * 80);
        for (final Main main : values())
            builder.append(main.usage()).append('\n');
        err.print(builder.toString());
    }

    String usage() { return message("usage", Main.class.getSimpleName()); }
    String help() { return message("help"); }

    String message(String key, Object... args) {
        return String.format(
                ResourceBundle
                    .getBundle(Main.class.getCanonicalName())
                    .getString(name() + "." + key),
                args);
    }
}
