#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import java.io.*;
import static java.io.File.*;
import java.util.*;
import net.java.truelicense.core.io.*;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 * Forks Java processes to run the Main class and test the license key life
 * cycle.
 * <strong>Note:</strong> If you changed the licensing subject when generating
 * this project from the TrueLicense Maven Archeype, then you need to update
 * the license key in {@code ${basedir}/src/test/resources/license.key}.
 * Otherwise, these tests will fail.
 *
 * @author Christian Schlichtherle
 */
public class MainIT {

    private static final String JAVA =
            System.getProperty("java.home") + separator +
            "bin" + separator +
            "java";

    private static final String CLASS_PATH =
            System.getProperty("java.class.path");

    @Test
    public void testLicenseKeyLifeCycle() throws Exception {
        testCreate();
        try {
            testView();
            testVerify();
        } finally {
            testUninstall();
        }
    }

    private static void testCreate() throws Exception {
        final File key = File.createTempFile("key", null);
        try {
            Source source = Sources.forResource("license.key",
                    MainIT.class.getClassLoader());
            Copy.copy(source, new FileStore(key));
            main("install", key.getAbsolutePath());
        } finally {
            key.delete();
        }
    }

    private static void testView() throws Exception {
        main("view");
    }

    private static void testVerify() throws Exception {
        main("verify");
    }

    private static void testUninstall() throws Exception {
        main("uninstall");
    }

    private static void main(String... args) throws Exception {
        final List<String> cmd = new ArrayList<String>();
        cmd.addAll(Arrays.asList(JAVA, "-cp", CLASS_PATH, Main.class.getName()));
        cmd.addAll(Arrays.asList(args));
        final Process proc = new ProcessBuilder(cmd).start();
        try {
            cat(proc.getErrorStream(), System.err);
        } finally {
            proc.waitFor();
        }
        assertEquals(0, proc.exitValue());
    }

    private static void cat(final InputStream in, final OutputStream out) throws IOException {
        final byte[] buffer = new byte[Store.BUFSIZE];
        int read;
        while (0 <= (read = in.read(buffer)))
            out.write(buffer, 0, read);
    }

    private static byte[] read(File file) throws IOException {
        final byte[] data = new byte[(int) file.length()];
        final InputStream in = new FileInputStream(file);
        try {
            new DataInputStream(in).readFully(data);
        } finally {
            in.close();
        }
        return data;
    }
}
