#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import java.io.File;
import static java.lang.System.*;
import java.util.*;
import javax.annotation.concurrent.Immutable;
import net.java.truelicense.core.*;
import net.java.truelicense.core.codec.*;
import net.java.truelicense.core.io.*;
import static net.java.truelicense.obfuscate.ObfuscatedString.*;

/**
 * Creates license keys for ${subject} or obfuscates strings for configuring
 * the licensing schema.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public enum Main {

    USAGE {
        @Override protected void run(final Deque<String> params) {
            throw new IllegalArgumentException();
        }
    },

    CREATE {
        @Override protected void run(Deque<String> params) throws Exception {
            final Map<CreateOption, String>
                    o = parse(params, CreateOption.class);
            String lkp = params.poll(); // <license-key-path>
            parse(params, CreateOption.class, o);
            if (!params.isEmpty()) throw new IllegalArgumentException();
            if (null == lkp) lkp = o.get(CreateOption.KEY);
            final Store s = null != lkp
                    ? vc.file(new File(lkp))
                    : new MemoryStore(); // store to /dev/null
            final Codec c = vc.codec(License.class);
            final License il;
            final String ilp = o.get(CreateOption.INPUT); // <input-license-path>
            il = null == ilp
                    ? vc.license()
                    : (License) c.decode("-".equals(ilp)
                        ? Sources.input()
                        : vc.file(new File(ilp)));
            final License ol = vm.create(il, s);
            err.println(ol);
            final String olp = o.get(CreateOption.OUTPUT); // <output-license-path>
            if (null != olp) c.encode("-".equals(olp)
                    ? Sinks.output()
                    : vc.file(new File(olp)),
                    ol);
        }
    },

    HELP {
        @Override protected void run(final Deque<String> params) {
            out.println(Main.valueOf(params.pop().toUpperCase(Locale.ROOT)).help());
        }
    },

    OBFUSCATE {
        @Override protected void run(Deque<String> params) {
            for (   String param;
                    null != (param = params.poll());
                    ) {
                out.println(obfuscate(param));
            }
        }
    },

    VERSION {
        @Override protected void run(final Deque<String> params) {
            out.println(message("version", Main.class.getSimpleName()));
        }
    };

    private static final LicenseVendorContext vc = LicensingSchema.context();
    private static final LicenseVendorManager vm = LicensingSchema.manager();

    /**
     * Runs this command.
     * Implementations are free to modify the given deque.
     *
     * @param  params the command parameters.
     */
    // This method needs to be protected or otherwise ProGuard 4.8 may generate
    // invalid code when --repackageclasses is set, even if
    // -allowaccessmodification is set, too.
    // See https://sourceforge.net/p/proguard/bugs/367/ for details.
    protected abstract void run(Deque<String> params) throws Exception;

    @SuppressWarnings("CallToThreadDumpStack")
    public static void main(final String[] args) {
        int status;
        try {
            final Deque<String> p = new LinkedList<String>(Arrays.asList(args));
            final String cmd = p.pop().toUpperCase(Locale.ROOT);
            valueOf(cmd).run(p);
            status = 0;
        } catch (final IllegalArgumentException ex) {
            errUsage();
            status = 1;
        } catch (final NoSuchElementException ex) {
            errUsage();
            status = 1;
        } catch (Exception ex) {
            ex.printStackTrace();
            status = 2;
        }
        System.exit(status);
    }

    private static void errUsage() {
        final StringBuilder builder = new StringBuilder(25 * 80);
        for (final Main main : values())
            builder.append(main.usage()).append('\n');
        err.print(builder.toString());
    }

    String usage() { return message("usage", Main.class.getSimpleName()); }
    String help() { return message("help"); }

    String message(String key, Object... args) {
        return String.format(
                ResourceBundle
                    .getBundle(Main.class.getCanonicalName())
                    .getString(name() + "." + key),
                args);
    }

    /**
     * Parses the given command parameters for options of the given class with
     * a string parameter.
     * As a side effect, any found options are popped off the parameter stack.
     *
     * @param  <T> the type of the enum class for the options.
     * @param  params the command parameters.
     * @param  type the enum class for the options.
     * @return an enum map with the options and parameters found.
     */
    private static <T extends Enum<T>> EnumMap<T, String> parse(
            final Deque<String> params,
            final Class<T> type) {
        return parse(params, type, new EnumMap<T, String>(type));
    }

    private static <T extends Enum<T>, M extends Map<T, String>> M parse(
            final Deque<String> params,
            final Class<T> type,
            final M options) {
        for (   String param;
                null != (param = params.peek()) && '-' == param.charAt(0);
                ) {
            params.pop(); // consume
            param = param.substring(1).toUpperCase(Locale.ROOT);
            options.put(valueOf(type, param), params.pop());
        }
        return options;
    }

    private enum CreateOption { KEY, INPUT, OUTPUT }
}
