#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import java.io.*;
import static java.io.File.*;
import java.util.*;
import net.java.truelicense.core.io.Store;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 * @author Christian Schlichtherle
 */
public class MainIT {

    private static final String JAVA =
            System.getProperty("java.home") + separator +
            "bin" + separator +
            "java";

    private static final String CLASS_PATH =
            System.getProperty("java.class.path");

    @Test
    public void testCreate() throws Exception {
        final File input = File.createTempFile("input", null);
        input.delete();
        try {
            create("-output", input.getAbsolutePath());
            final File output = File.createTempFile("output", null);
            output.delete();
            try {
                create( "-input",  input.getAbsolutePath(),
                        "-output", output.getAbsolutePath());
                assertArrayEquals(read(input), read(output));
            } finally {
                output.delete();
            }
        } finally {
            input.delete();
        }
    }

    private static void create(String... options) throws Exception {
        final File key = File.createTempFile("key", null);
        key.delete();
        try {
            final List<String> cmd = new ArrayList<String>();
            cmd.addAll(Arrays.asList(
                    JAVA, "-cp", CLASS_PATH, Main.class.getName(),
                    "create", "-key", key.getAbsolutePath()));
            cmd.addAll(Arrays.asList(options));
            final Process proc = new ProcessBuilder(cmd).start();
            try {
                cat(proc.getErrorStream(), System.err);
            } finally {
                proc.waitFor();
            }
            assertEquals(0, proc.exitValue());
            assertTrue(key.isFile());
        } finally {
            key.delete();
        }
    }

    private static void cat(final InputStream in, final OutputStream out) throws IOException {
        final byte[] buffer = new byte[Store.BUFSIZE];
        int read;
        while (0 <= (read = in.read(buffer)))
            out.write(buffer, 0, read);
    }

    private static byte[] read(File file) throws IOException {
        final byte[] data = new byte[(int) file.length()];
        final InputStream in = new FileInputStream(file);
        try {
            new DataInputStream(in).readFully(data);
        } finally {
            in.close();
        }
        return data;
    }
}
